@@ -13,8 +13,9 @@
 #include <initializer_list>
 #include <utility>
 
-#include "xtl/xsequence.hpp"
+#include <xtl/xsequence.hpp>
 
+#include "xbuffer_adaptor.hpp"
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
@@ -28,7 +29,7 @@ namespace xt
     template <class EC, layout_type L, class SC, class Tag>
     struct xcontainer_inner_types<xarray_container<EC, L, SC, Tag>>
     {
-        using container_type = EC;
+        using storage_type = EC;
         using shape_type = SC;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
@@ -67,7 +68,7 @@ namespace xt
         using self_type = xarray_container<EC, L, SC, Tag>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
-        using container_type = typename base_type::container_type;
+        using storage_type = typename base_type::storage_type;
         using allocator_type = typename base_type::allocator_type;
         using value_type = typename base_type::value_type;
         using reference = typename base_type::reference;
@@ -87,7 +88,7 @@ namespace xt
         explicit xarray_container(const shape_type& shape, const_reference value, layout_type l = L);
         explicit xarray_container(const shape_type& shape, const strides_type& strides);
         explicit xarray_container(const shape_type& shape, const strides_type& strides, const_reference value);
-        explicit xarray_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides);
+        explicit xarray_container(storage_type&& storage, inner_shape_type&& shape, inner_strides_type&& strides);
 
         xarray_container(const value_type& t);
         xarray_container(nested_initializer_list_t<value_type, 1> t);
@@ -115,10 +116,10 @@ namespace xt
 
     private:
 
-        container_type m_data;
+        storage_type m_storage;
 
-        container_type& data_impl() noexcept;
-        const container_type& data_impl() const noexcept;
+        storage_type& storage_impl() noexcept;
+        const storage_type& storage_impl() const noexcept;
 
         friend class xcontainer<xarray_container<EC, L, SC, Tag>>;
     };
@@ -130,14 +131,14 @@ namespace xt
     template <class EC, layout_type L, class SC, class Tag>
     struct xcontainer_inner_types<xarray_adaptor<EC, L, SC, Tag>>
     {
-        using container_type = std::remove_reference_t<EC>;
+        using storage_type = std::remove_reference_t<EC>;
         using shape_type = SC;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray_container<container_type, L, SC, Tag>;
+        using temporary_type = xarray_container<temporary_container_t<storage_type>, L, SC, Tag>;
         static constexpr layout_type layout = L;
     };
 
@@ -173,21 +174,22 @@ namespace xt
         using self_type = xarray_adaptor<EC, L, SC, Tag>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
-        using container_type = typename base_type::container_type;
+        using storage_type = typename base_type::storage_type;
+        using allocator_type = typename base_type::allocator_type;
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
         using backstrides_type = typename base_type::backstrides_type;
         using temporary_type = typename semantic_base::temporary_type;
         using expression_tag = Tag;
 
-        xarray_adaptor(container_type&& data);
-        xarray_adaptor(const container_type& data);
+        xarray_adaptor(storage_type&& storage);
+        xarray_adaptor(const storage_type& storage);
 
         template <class D>
-        xarray_adaptor(D&& data, const shape_type& shape, layout_type l = L);
+        xarray_adaptor(D&& storage, const shape_type& shape, layout_type l = L);
 
         template <class D>
-        xarray_adaptor(D&& data, const shape_type& shape, const strides_type& strides);
+        xarray_adaptor(D&& storage, const shape_type& shape, const strides_type& strides);
 
         ~xarray_adaptor() = default;
 
@@ -203,10 +205,10 @@ namespace xt
 
     private:
 
-        container_closure_type m_data;
+        container_closure_type m_storage;
 
-        container_type& data_impl() noexcept;
-        const container_type& data_impl() const noexcept;
+        storage_type& storage_impl() noexcept;
+        const storage_type& storage_impl() const noexcept;
 
 
         friend class xcontainer<xarray_adaptor<EC, L, SC, Tag>>;
@@ -225,7 +227,7 @@ namespace xt
      */
     template <class EC, layout_type L, class SC, class Tag>
     inline xarray_container<EC, L, SC, Tag>::xarray_container()
-        : base_type(), m_data(1, value_type())
+        : base_type(), m_storage(1, value_type())
     {
     }
 
@@ -239,7 +241,7 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
@@ -253,8 +255,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const_reference value, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
-        std::fill(m_data.begin(), m_data.end(), value);
+        base_type::resize(shape, l);
+        std::fill(m_storage.begin(), m_storage.end(), value);
     }
 
     /**
@@ -266,7 +268,7 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
 
     /**
@@ -280,8 +282,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
-        base_type::reshape(shape, strides);
-        std::fill(m_data.begin(), m_data.end(), value);
+        base_type::resize(shape, strides);
+        std::fill(m_storage.begin(), m_storage.end(), value);
     }
 
     /**
@@ -293,20 +295,20 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(const value_type& t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t), true);
-        nested_copy(m_data.begin(), t);
+        base_type::resize(xt::shape<shape_type>(t), true);
+        nested_copy(m_storage.begin(), t);
     }
 
     /**
      * Allocates an xarray_container by moving specified data, shape and strides
      *
-     * @param data the data for the xarray_container
+     * @param storage the data for the xarray_container
      * @param shape the shape of the xarray_container
      * @param strides the strides of the xarray_container
      */
     template <class EC, layout_type L, class SC, class Tag>
-    inline xarray_container<EC, L, SC, Tag>::xarray_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides)
-        : base_type(std::move(shape), std::move(strides)), m_data(std::move(data))
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(storage_type&& storage, inner_shape_type&& shape, inner_strides_type&& strides)
+        : base_type(std::move(shape), std::move(strides)), m_storage(std::move(storage))
     {
     }
     //@}
@@ -323,8 +325,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 1> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
@@ -335,8 +337,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 2> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
@@ -347,8 +349,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 3> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
@@ -359,8 +361,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 4> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
@@ -371,8 +373,8 @@ namespace xt
     inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 5> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
     //@}
 
@@ -401,10 +403,10 @@ namespace xt
         : base_type()
     {
         // Avoids unintialized data because of (m_shape == shape) condition
-        // in reshape (called by assign), which is always true when dimension == 0.
+        // in resize (called by assign), which is always true when dimension == 0.
         if (e.derived_cast().dimension() == 0)
         {
-            m_data.resize(1);
+            detail::resize_data_container(m_storage, std::size_t(1));
         }
         semantic_base::assign(e);
     }
@@ -421,15 +423,15 @@ namespace xt
     //@}
 
     template <class EC, layout_type L, class SC, class Tag>
-    inline auto xarray_container<EC, L, SC, Tag>::data_impl() noexcept -> container_type&
+    inline auto xarray_container<EC, L, SC, Tag>::storage_impl() noexcept -> storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     template <class EC, layout_type L, class SC, class Tag>
-    inline auto xarray_container<EC, L, SC, Tag>::data_impl() const noexcept -> const container_type&
+    inline auto xarray_container<EC, L, SC, Tag>::storage_impl() const noexcept -> const storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     /******************
@@ -442,52 +444,52 @@ namespace xt
     //@{
     /**
      * Constructs an xarray_adaptor of the given stl-like container.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      */
     template <class EC, layout_type L, class SC, class Tag>
-    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(container_type&& data)
-        : base_type(), m_data(std::move(data))
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(storage_type&& storage)
+        : base_type(), m_storage(std::move(storage))
     {
     }
 
     /**
      * Constructs an xarray_adaptor of the given stl-like container.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      */
     template <class EC, layout_type L, class SC, class Tag>
-    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(const container_type& data)
-        : base_type(), m_data(data)
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(const storage_type& storage)
+        : base_type(), m_storage(storage)
     {
     }
 
     /**
      * Constructs an xarray_adaptor of the given stl-like container,
      * with the specified shape and layout_type.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      * @param shape the shape of the xarray_adaptor
      * @param l the layout_type of the xarray_adaptor
      */
     template <class EC, layout_type L, class SC, class Tag>
     template <class D>
-    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& data, const shape_type& shape, layout_type l)
-        : base_type(), m_data(std::forward<D>(data))
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& storage, const shape_type& shape, layout_type l)
+        : base_type(), m_storage(std::forward<D>(storage))
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
      * Constructs an xarray_adaptor of the given stl-like container,
      * with the specified shape and strides.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      * @param shape the shape of the xarray_adaptor
      * @param strides the strides of the xarray_adaptor
      */
     template <class EC, layout_type L, class SC, class Tag>
     template <class D>
-    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& data, const shape_type& shape, const strides_type& strides)
-        : base_type(), m_data(std::forward<D>(data))
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& storage, const shape_type& shape, const strides_type& strides)
+        : base_type(), m_storage(std::forward<D>(storage))
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
     //@}
 
@@ -495,7 +497,7 @@ namespace xt
     inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(const xarray_adaptor& rhs) -> self_type&
     {
         base_type::operator=(rhs);
-        m_data = rhs.m_data;
+        m_storage = rhs.m_storage;
         return *this;
     }
 
@@ -503,7 +505,7 @@ namespace xt
     inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(xarray_adaptor&& rhs) -> self_type&
     {
         base_type::operator=(std::move(rhs));
-        m_data = rhs.m_data;
+        m_storage = rhs.m_storage;
         return *this;
     }
 
@@ -513,7 +515,7 @@ namespace xt
         base_type::shape_impl() = std::move(const_cast<shape_type&>(rhs.shape()));
         base_type::strides_impl() = std::move(const_cast<strides_type&>(rhs.strides()));
         base_type::backstrides_impl() = std::move(const_cast<backstrides_type&>(rhs.backstrides()));
-        m_data = std::move(rhs.data());
+        m_storage = std::move(rhs.storage());
         return *this;
     }
 
@@ -533,15 +535,15 @@ namespace xt
     //@}
 
     template <class EC, layout_type L, class SC, class Tag>
-    inline auto xarray_adaptor<EC, L, SC, Tag>::data_impl() noexcept -> container_type&
+    inline auto xarray_adaptor<EC, L, SC, Tag>::storage_impl() noexcept -> storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     template <class EC, layout_type L, class SC, class Tag>
-    inline auto xarray_adaptor<EC, L, SC, Tag>::data_impl() const noexcept -> const container_type&
+    inline auto xarray_adaptor<EC, L, SC, Tag>::storage_impl() const noexcept -> const storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 }
 
