@@ -19,13 +19,7 @@
 
 #include "xexpression.hpp"
 #include "xmath.hpp"
-#include "xview.hpp"
-
-#ifdef _WIN32
-    using precision_type = typename std::streamsize;
-#else
-    using precision_type = int;
-#endif
+#include "xstrided_view.hpp"
 
 namespace xt
 {
@@ -40,7 +34,7 @@ namespace xt
             std::size_t edgeitems = 3;
             std::size_t line_width = 75;
             std::size_t threshold = 1000;
-            precision_type precision = -1;  // default precision
+            std::streamsize precision = -1;  // default precision
         };
 
         inline print_options_impl& print_options()
@@ -88,7 +82,7 @@ namespace xt
          *
          * @param precision The number of digits for floating point output
          */
-        inline void set_precision(precision_type precision)
+        inline void set_precision(std::streamsize precision)
         {
             print_options().precision = precision;
         }
@@ -100,131 +94,100 @@ namespace xt
 
     namespace detail
     {
-        template <std::size_t I>
-        struct xout
+        template <class E, class F>
+        std::ostream& xoutput(std::ostream& out, const E& e, slice_vector& slices, F& printer, std::size_t blanks,
+                              std::streamsize element_width, std::size_t edgeitems, std::size_t line_width)
         {
-            template <class E, class F>
-            static std::ostream& output(std::ostream& out, const E& e, F& printer, std::size_t blanks,
-                                        precision_type element_width, std::size_t edgeitems, std::size_t line_width)
+            using size_type = typename E::size_type;
+
+            const auto view = xt::strided_view(e, slices);
+            if (view.dimension() == 0)
             {
-                using size_type = typename E::size_type;
+                printer.print_next(out);
+            }
+            else
+            {
+                std::string indents(blanks, ' ');
 
-                if (e.dimension() == 0)
+                size_type i = 0;
+                size_type elems_on_line = 0;
+                size_type ewp2 = static_cast<size_type>(element_width) + size_type(2);
+                size_type line_lim = static_cast<size_type>(std::floor(line_width / ewp2));
+
+                out << '{';
+                for (; i != view.shape()[0] - 1; ++i)
                 {
-                    printer.print_next(out);
-                }
-                else
-                {
-                    std::string indents(blanks, ' ');
-
-                    size_type i = 0;
-                    size_type elems_on_line = 0;
-                    size_type ewp2 = static_cast<size_type>(element_width) + size_type(2);
-                    size_type line_lim = static_cast<size_type>(std::floor(line_width / ewp2));
-
-                    out << '{';
-                    for (; i != e.shape()[0] - 1; ++i)
+                    if (edgeitems && view.shape()[0] > (edgeitems * 2) && i == edgeitems)
                     {
-                        if (edgeitems && e.shape()[0] > (edgeitems * 2) && i == edgeitems)
+                        out << "..., ";
+                        if (view.dimension() > 1)
                         {
-                            out << "..., ";
-                            if (e.dimension() > 1)
-                            {
-                                elems_on_line = 0;
-                                out << std::endl
-                                    << indents;
-                            }
-                            i = e.shape()[0] - edgeitems;
-                        }
-                        if (e.dimension() == 1 && line_lim != 0 && elems_on_line >= line_lim)
-                        {
-                            out << std::endl
-                                << indents;
                             elems_on_line = 0;
-                        }
-
-                        xout<I - 1>::output(out, view(e, i), printer, blanks + 1, element_width, edgeitems, line_width) << ',';
-
-                        elems_on_line++;
-
-                        if (I == 1 || e.dimension() == 1)
-                        {
-                            out << ' ';
-                        }
-                        else
-                        {
                             out << std::endl
                                 << indents;
                         }
+                        i = view.shape()[0] - edgeitems;
                     }
-                    if (e.dimension() == 1 && line_lim != 0 && elems_on_line >= line_lim)
+                    if (view.dimension() == 1 && line_lim != 0 && elems_on_line >= line_lim)
+                    {
+                        out << std::endl
+                            << indents;
+                        elems_on_line = 0;
+                    }
+                    slices.push_back(static_cast<int>(i));
+                    xoutput(out, e, slices, printer, blanks + 1, element_width, edgeitems, line_width) << ',';
+                    slices.pop_back();
+                    elems_on_line++;
+
+                    if (view.dimension() == 1)
+                    {
+                        out << ' ';
+                    }
+                    else
                     {
                         out << std::endl
                             << indents;
                     }
-                    xout<I - 1>::output(out, view(e, i), printer, blanks + 1, element_width, edgeitems, line_width) << '}';
                 }
-                return out;
+                if (view.dimension() == 1 && line_lim != 0 && elems_on_line >= line_lim)
+                {
+                    out << std::endl
+                        << indents;
+                }
+                slices.push_back(static_cast<int>(i));
+                xoutput(out, e, slices, printer, blanks + 1, element_width, edgeitems, line_width) << '}';
+                slices.pop_back();
             }
-        };
+            return out;
+        }
 
-        template <>
-        struct xout<0>
+        template <class F, class E>
+        static void recurser_run(F& fn, const E& e, slice_vector& slices, std::size_t lim = 0)
         {
-            template <class E, class F>
-            static std::ostream& output(std::ostream& out, const E& e, F& printer,
-                                        std::size_t, precision_type, std::size_t, std::size_t)
+            using size_type = typename E::size_type;
+            const auto view = strided_view(e, slices);
+            if (view.dimension() == 0)
             {
-                if (e.dimension() == 0)
-                {
-                    return printer.print_next(out);
-                }
-                else
-                {
-                    return out << "{...}";
-                }
+                fn.update(view());
             }
-        };
-
-        template <std::size_t I>
-        struct recurser
-        {
-            template <class F, class E>
-            static void run(F& fn, const E& e, std::size_t lim = 0)
+            else
             {
-                using size_type = typename E::size_type;
-                if (e.dimension() == 0)
+                size_type i = 0;
+                for (; i != view.shape()[0] - 1; ++i)
                 {
-                    fn.update(e());
-                }
-                else
-                {
-                    size_type i = 0;
-                    for (; i != e.shape()[0] - 1; ++i)
+                    if (lim && view.shape()[0] > (lim * 2) && i == lim)
                     {
-                        if (lim && e.shape()[0] > (lim * 2) && i == lim)
-                        {
-                            i = e.shape()[0] - lim;
-                        }
-                        recurser<I - 1>::run(fn, view(e, i), lim);
+                        i = view.shape()[0] - lim;
                     }
-                    recurser<I - 1>::run(fn, view(e, i), lim);
+                    slices.push_back(static_cast<int>(i));
+                    recurser_run(fn, e, slices, lim);
+                    slices.pop_back();
                 }
+                slices.push_back(static_cast<int>(i));
+                recurser_run(fn, e, slices, lim);
+                slices.pop_back();
             }
-        };
-
-        template <>
-        struct recurser<0>
-        {
-            template <class F, class E>
-            static void run(F& fn, const E& e, std::size_t)
-            {
-                if (e.dimension() == 0)
-                {
-                    fn.update(e());
-                }
-            }
-        };
+        }
 
         template <class T, class E = void>
         struct printer;
@@ -232,11 +195,11 @@ namespace xt
         template <class T>
         struct printer<T, std::enable_if_t<std::is_floating_point<typename T::value_type>::value>>
         {
-            using value_type = typename T::value_type;
+            using value_type = std::decay_t<typename T::value_type>;
             using cache_type = std::vector<value_type>;
             using cache_iterator = typename cache_type::const_iterator;
 
-            printer(precision_type precision)
+            explicit printer(std::streamsize precision)
                 : m_precision(precision)
             {
             }
@@ -257,10 +220,10 @@ namespace xt
                 }
                 else
                 {
-                    precision_type decimals = 1;  // print a leading 0
+                    std::streamsize decimals = 1;  // print a leading 0
                     if (std::floor(m_max) != 0)
                     {
-                        decimals += (precision_type)std::log10(std::floor(m_max));
+                        decimals += std::streamsize(std::log10(std::floor(m_max)));
                     }
                     // 2 => sign and dot
                     m_width = 2 + decimals + m_precision;
@@ -276,7 +239,10 @@ namespace xt
                 if (!m_scientific)
                 {
                     std::stringstream buf;
-                    buf << std::setw(m_width) << std::fixed << std::setprecision(m_precision) << (*m_it);
+                    buf.width(m_width);
+                    buf << std::fixed;
+                    buf.precision(m_precision);
+                    buf << (*m_it);
                     if (!m_required_precision)
                     {
                         buf << '.';
@@ -294,12 +260,17 @@ namespace xt
                 {
                     if (!m_large_exponent)
                     {
-                        out << std::scientific << std::setw(m_width) << (*m_it);
+                        out << std::scientific;
+                        out.width(m_width);
+                        out << (*m_it);
                     }
                     else
                     {
                         std::stringstream buf;
-                        buf << std::setw(m_width) << std::scientific << std::setprecision(m_precision) << (*m_it);
+                        buf.width(m_width);
+                        buf << std::scientific;
+                        buf.precision(m_precision);
+                        buf << (*m_it);
                         std::string res = buf.str();
 
                         if (res[res.size() - 4] == 'e')
@@ -320,7 +291,7 @@ namespace xt
                 {
                     if (!m_scientific || !m_large_exponent)
                     {
-                        int exponent = 1 + (int)std::log10(math::abs(val));
+                        int exponent = 1 + int(std::log10(math::abs(val)));
                         if (exponent <= -5 || exponent > 7)
                         {
                             m_scientific = true;
@@ -346,7 +317,7 @@ namespace xt
                 m_cache.push_back(val);
             }
 
-            precision_type width()
+            std::streamsize width()
             {
                 return m_width;
             }
@@ -355,9 +326,9 @@ namespace xt
 
             bool m_large_exponent = false;
             bool m_scientific = false;
-            precision_type m_width = 9;
-            precision_type m_precision;
-            precision_type m_required_precision = 0;
+            std::streamsize m_width = 9;
+            std::streamsize m_precision;
+            std::streamsize m_required_precision = 0;
             value_type m_max = 0;
 
             cache_type m_cache;
@@ -367,25 +338,26 @@ namespace xt
         template <class T>
         struct printer<T, std::enable_if_t<std::is_integral<typename T::value_type>::value && !std::is_same<typename T::value_type, bool>::value>>
         {
-            using value_type = typename T::value_type;
+            using value_type = std::decay_t<typename T::value_type>;
             using cache_type = std::vector<value_type>;
             using cache_iterator = typename cache_type::const_iterator;
 
-            printer(precision_type)
+            explicit printer(std::streamsize)
             {
             }
 
             void init()
             {
                 m_it = m_cache.cbegin();
-                m_width = 1 + (precision_type)std::log10(m_max) + m_sign;
+                m_width = 1 + std::streamsize(std::log10(m_max)) + m_sign;
             }
 
             std::ostream& print_next(std::ostream& out)
             {
                 // + enables printing of chars etc. as numbers
                 // TODO should chars be printed as numbers?
-                out << std::setw(m_width) << +(*m_it);
+                out.width(m_width);
+                out << +(*m_it);
                 ++m_it;
                 return out;
             }
@@ -403,14 +375,14 @@ namespace xt
                 m_cache.push_back(val);
             }
 
-            precision_type width()
+            std::streamsize width()
             {
                 return m_width;
             }
 
         private:
 
-            precision_type m_width;
+            std::streamsize m_width;
             bool m_sign = false;
             value_type m_max = 0;
 
@@ -425,7 +397,7 @@ namespace xt
             using cache_type = std::vector<bool>;
             using cache_iterator = typename cache_type::const_iterator;
 
-            printer(precision_type)
+            explicit printer(std::streamsize)
             {
             }
 
@@ -444,8 +416,8 @@ namespace xt
                 {
                     out << "false";
                 }
-                // the following std::setw(5) isn't working correctly on OSX.
-                // out << std::boolalpha << std::setw(m_width) << (*m_it);
+                // TODO: the following std::setw(5) isn't working correctly on OSX.
+                //out << std::boolalpha << std::setw(m_width) << (*m_it);
                 ++m_it;
                 return out;
             }
@@ -455,14 +427,14 @@ namespace xt
                 m_cache.push_back(val);
             }
 
-            precision_type width()
+            std::streamsize width()
             {
                 return m_width;
             }
 
         private:
 
-            precision_type m_width = 5;
+            std::streamsize m_width = 5;
 
             cache_type m_cache;
             cache_iterator m_it;
@@ -471,11 +443,11 @@ namespace xt
         template <class T>
         struct printer<T, std::enable_if_t<xtl::is_complex<typename T::value_type>::value>>
         {
-            using value_type = typename T::value_type;
+            using value_type = std::decay_t<typename T::value_type>;
             using cache_type = std::vector<bool>;
             using cache_iterator = typename cache_type::const_iterator;
 
-            printer(precision_type precision)
+            explicit printer(std::streamsize precision)
                 : real_printer(precision), imag_printer(precision)
             {
             }
@@ -520,7 +492,7 @@ namespace xt
                 m_signs.push_back(std::signbit(val.imag()));
             }
 
-            precision_type width()
+            std::streamsize width()
             {
                 return real_printer.width() + imag_printer.width() + 2;
             }
@@ -535,11 +507,11 @@ namespace xt
         template <class T>
         struct printer<T, std::enable_if_t<!std::is_fundamental<typename T::value_type>::value && !xtl::is_complex<typename T::value_type>::value>>
         {
-            using value_type = typename T::value_type;
+            using value_type = std::decay_t<typename T::value_type>;
             using cache_type = std::vector<std::string>;
             using cache_iterator = typename cache_type::const_iterator;
 
-            printer(precision_type)
+            explicit printer(std::streamsize)
             {
             }
 
@@ -554,7 +526,8 @@ namespace xt
 
             std::ostream& print_next(std::ostream& out)
             {
-                out << std::setw(m_width) << *m_it;
+                out.width(m_width);
+                out << *m_it;
                 ++m_it;
                 return out;
             }
@@ -566,19 +539,19 @@ namespace xt
                 std::string s = buf.str();
                 if (int(s.size()) > m_width)
                 {
-                    m_width = int(s.size());
+                    m_width = std::streamsize(s.size());
                 }
                 m_cache.push_back(s);
             }
 
-            precision_type width()
+            std::streamsize width()
             {
                 return m_width;
             }
 
         private:
 
-            precision_type m_width = 0;
+            std::streamsize m_width = 0;
             cache_type m_cache;
             cache_iterator m_it;
         };
@@ -586,7 +559,7 @@ namespace xt
         template <class E>
         struct custom_formatter
         {
-            using value_type = typename E::value_type;
+            using value_type = std::decay_t<typename E::value_type>;
 
             template <class F>
             custom_formatter(F&& func)
@@ -603,21 +576,6 @@ namespace xt
 
             std::function<std::string(const value_type&)> m_func;
         };
-
-        template <class S>
-        struct recursion_depth
-        {
-            static constexpr std::size_t value = 5;
-        };
-
-// Note: std::min is not constexpr on old versions of gcc (4.x) and clang.
-#define XTENSOR_MIN(x, y) (x > y ? y : x)
-        template <class T, std::size_t N>
-        struct recursion_depth<std::array<T, N>>
-        {
-            static constexpr std::size_t value = XTENSOR_MIN(5, N);
-        };
-#undef XTENSOR_MIN
     }
 
     template <class E, class F>
@@ -644,22 +602,23 @@ namespace xt
             return out;
         }
 
-        precision_type temp_precision = (precision_type)out.precision();
-        precision_type precision = temp_precision;
+        auto temp_precision = out.precision();
+        auto precision = temp_precision;
         if (print_options::print_options().precision != -1)
         {
-            out << std::setprecision(print_options::print_options().precision);
+            out.precision(print_options::print_options().precision);
             precision = print_options::print_options().precision;
         }
 
         detail::printer<E> p(precision);
 
-        constexpr std::size_t depth = detail::recursion_depth<typename E::shape_type>::value;
-        detail::recurser<depth>::run(p, d, lim);
+        slice_vector sv;
+        detail::recurser_run(p, d, sv, lim);
         p.init();
-        detail::xout<depth>::output(out, d, p, 1, p.width(), lim, print_options::print_options().line_width);
+        sv.clear();
+        xoutput(out, d, sv, p, 1, p.width(), lim, print_options::print_options().line_width);
 
-        out << std::setprecision(temp_precision);  // restore precision
+        out.precision(temp_precision);  // restore precision
 
         return out;
     }
