@@ -21,7 +21,7 @@
 
 namespace xt
 {
-/*************************************
+    /*************************************
      * norm functions for built-in types *
      *************************************/
 
@@ -141,7 +141,7 @@ namespace xt
     template <class T>
     inline auto norm_linf(const std::complex<T>& t) noexcept
     {
-        return std::max(std::abs(t.real()), std::abs(t.imag()));
+        return (std::max)(std::abs(t.real()), std::abs(t.imag()));
     }
 
     /**
@@ -223,7 +223,7 @@ namespace xt
     XTENSOR_NORM_FUNCTION(norm_l0, unsigned long long, XTENSOR_EMPTY, +, std::plus)
     XTENSOR_NORM_FUNCTION(norm_l1, big_promote_type_t<value_type>, XTENSOR_EMPTY, +, std::plus)
     XTENSOR_NORM_FUNCTION(norm_sq, big_promote_type_t<value_type>, XTENSOR_EMPTY, +, std::plus)
-    XTENSOR_NORM_FUNCTION(norm_linf, decltype(norm_linf(*(value_type*)0)), std::max<result_type>, XTENSOR_COMMA, math::maximum)
+    XTENSOR_NORM_FUNCTION(norm_linf, decltype(norm_linf(std::declval<value_type>())), (std::max<result_type>), XTENSOR_COMMA, math::maximum)
 
 #undef XTENSOR_EMPTY
 #undef XTENSOR_COMMA
