@@ -21,8 +21,8 @@
 #include <vector>
 #endif
 
-#include "xtl/xfunctional.hpp"
-#include "xtl/xsequence.hpp"
+#include <xtl/xfunctional.hpp>
+#include <xtl/xsequence.hpp>
 
 #include "xbuilder.hpp"
 #include "xexpression.hpp"
@@ -39,52 +39,87 @@ namespace xt
 
 #define DEFAULT_STRATEGY_REDUCERS evaluation_strategy::lazy
 
-    template <class F, class E, class X, class ES = DEFAULT_STRATEGY_REDUCERS,
+    template <class F, class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
               class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value, int>>
-    auto reduce(F&& f, E&& e, X&& axes, ES es = ES()) noexcept;
+    auto reduce(F&& f, E&& e, X&& axes, EVS es = EVS());
 
-    template <class F, class E, class ES = DEFAULT_STRATEGY_REDUCERS,
-              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, ES>::value, int>>
-    auto reduce(F&& f, E&& e, ES es = ES()) noexcept;
+    template <class F, class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
+              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, EVS>::value, int>>
+    auto reduce(F&& f, E&& e, EVS es = EVS());
 
 #ifdef X_OLD_CLANG
-    template <class F, class E, class I>
-    auto reduce(F&& f, E&& e, std::initializer_list<I> axes) noexcept;
+    template <class F, class E, class I, class EVS = DEFAULT_STRATEGY_REDUCERS>
+    auto reduce(F&& f, E&& e, std::initializer_list<I> axes, EVS es = EVS());
 #else
-    template <class F, class E, class I, std::size_t N, class ES = DEFAULT_STRATEGY_REDUCERS>
-    auto reduce(F&& f, E&& e, const I (&axes)[N], ES es = ES()) noexcept;
+    template <class F, class E, class I, std::size_t N, class EVS = DEFAULT_STRATEGY_REDUCERS>
+    auto reduce(F&& f, E&& e, const I (&axes)[N], EVS es = EVS());
 #endif
 
+    template <class ST, class X>
+    struct xreducer_shape_type;
+
+    template <class E, class X, class result_type>
+    struct xreducer_result_container
+    {
+        using type = xarray<result_type, std::decay_t<E>::static_layout>;
+    };
+
+    template <class T, std::size_t N, layout_type L, class A, class X, std::size_t NX, class result_type>
+    struct xreducer_result_container<xtensor<T, N, L, A>, std::array<X, NX>, result_type>
+    {
+        using type = xtensor<result_type, N - NX, L>;
+    };
+
+    template <class T, std::size_t... N, layout_type L, class X, std::size_t NX, class result_type>
+    struct xreducer_result_container<xtensor_fixed<T, xshape<N...>, L>, std::array<X, NX>, result_type>
+    {
+        using type = xtensor<result_type, sizeof...(N) - NX, L>;
+    };
+
     template <class F, class E, class X>
     auto reduce_immediate(F&& f, E&& e, X&& axes)
     {
-        using shape_type = std::vector<std::size_t>;
-        using accumulate_functor = std::decay_t<decltype(std::get<0>(f))>;
-        using result_type = typename accumulate_functor::result_type;
+        using shape_type = typename xreducer_shape_type<typename std::decay_t<E>::shape_type, std::decay_t<X>>::type;
+
+        using reduce_functor_type = typename std::decay_t<F>::reduce_functor_type;
+        using init_functor_type = typename std::decay_t<F>::init_functor_type;
+        using merge_functor_type = typename std::decay_t<F>::merge_functor_type;
+
+        using expr_value_type = typename std::decay_t<E>::value_type;
+
+        using result_type = std::decay_t<decltype(std::declval<reduce_functor_type>()(
+            std::declval<init_functor_type>()(std::declval<expr_value_type>()), std::declval<expr_value_type>()))>;
 
         // retrieve functors from triple struct
-        auto acc_fct = std::get<0>(f);
+        auto reduce_fct = std::get<0>(f);
         auto init_fct = std::get<1>(f);
         auto merge_fct = std::get<2>(f);
 
-        shape_type result_shape(e.dimension() - axes.size());
-        std::vector<std::size_t> iter_shape = e.shape();
-        std::vector<std::size_t> iter_strides(e.dimension());
+        shape_type result_shape;
+        resize_container(result_shape, e.dimension() - axes.size());
 
-        xt::xarray<result_type, std::decay_t<E>::static_layout> result;
+        dynamic_shape<std::size_t> iter_shape = xtl::forward_sequence<dynamic_shape<std::size_t>>(e.shape());
+        dynamic_shape<std::size_t> iter_strides(e.dimension());
+
+        using result_container_type = typename xreducer_result_container<std::decay_t<E>, X, result_type>::type;
+        result_container_type result;
 
         if (!std::is_sorted(axes.cbegin(), axes.cend()))
         {
             throw std::runtime_error("Reducing axes should be sorted");
         }
+        if (axes.size() != 0 && axes[axes.size() - 1] > e.dimension() - 1)
+        {
+            throw std::runtime_error("Axis " + std::to_string(axes[axes.size() - 1]) + " out of bounds for reduction.");
+        }
 
         // Fast track for complete reduction
         if (e.dimension() == axes.size())
         {
-            auto begin = e.data().begin();
+            auto begin = e.storage().begin();
             result_type tmp = init_fct(*begin);
             ++begin;
-            result(0) = std::accumulate(begin, e.data().end(), tmp, acc_fct);
+            result.data()[0] = std::accumulate(begin, e.storage().end(), tmp, reduce_fct);
             return result;
         }
 
@@ -99,23 +134,22 @@ namespace xt
             }
         }
 
-        result.reshape(result_shape, e.layout());
+        result.resize(result_shape, e.layout());
 
         std::size_t ax_idx = (e.layout() == layout_type::row_major) ? axes.size() - 1 : 0;
         std::size_t inner_loop_size = e.strides()[axes[ax_idx]];
-        std::size_t inner_stride    = e.strides()[axes[ax_idx]];
+        std::size_t inner_stride = e.strides()[axes[ax_idx]];
         std::size_t outer_loop_size = e.shape()[axes[ax_idx]];
 
         // The following code merges reduction axes "at the end" (or the beginning for col_major)
         // together by increasing the size of the outer loop where appropriate
-        auto merge_loops = [&outer_loop_size, &e](auto it, auto end)
-        {
+        auto merge_loops = [&outer_loop_size, &e](auto it, auto end) {
             auto last_ax = *it;
             ++it;
             for (; it != end; ++it)
             {
                 // note that we check is_sorted, so this condition is valid
-                if (std::abs(ptrdiff_t(*it) - ptrdiff_t(last_ax)) ==  1)
+                if (std::abs(std::ptrdiff_t(*it) - std::ptrdiff_t(last_ax)) == 1)
                 {
                     last_ax = *it;
                     outer_loop_size *= e.shape()[last_ax];
@@ -138,8 +172,8 @@ namespace xt
         {
             std::size_t last_ax = merge_loops(axes.rbegin(), axes.rend());
 
-            iter_shape.erase(iter_shape.begin() + (ptrdiff_t) last_ax, iter_shape.end());
-            iter_strides.erase(iter_strides.begin() + (ptrdiff_t) last_ax, iter_strides.end());
+            iter_shape.erase(iter_shape.begin() + std::ptrdiff_t(last_ax), iter_shape.end());
+            iter_strides.erase(iter_strides.begin() + std::ptrdiff_t(last_ax), iter_strides.end());
         }
         else if (e.layout() == layout_type::column_major)
         {
@@ -147,8 +181,8 @@ namespace xt
             std::size_t last_ax = merge_loops(axes.begin(), axes.end());
 
             // erasing the front vs the back
-            iter_shape.erase(iter_shape.begin(), iter_shape.begin() + (ptrdiff_t) last_ax + 1);
-            iter_strides.erase(iter_strides.begin(), iter_strides.begin() + (ptrdiff_t) last_ax + 1);
+            iter_shape.erase(iter_shape.begin(), iter_shape.begin() + std::ptrdiff_t(last_ax + 1));
+            iter_strides.erase(iter_strides.begin(), iter_strides.begin() + std::ptrdiff_t(last_ax + 1));
 
             // and reversing, to make it work with the same next_idx function
             std::reverse(iter_shape.begin(), iter_shape.end());
@@ -160,12 +194,11 @@ namespace xt
         }
 
         xindex temp_idx(iter_shape.size());
-        auto next_idx = [&iter_shape, &iter_strides, &temp_idx]()
-        {
+        auto next_idx = [&iter_shape, &iter_strides, &temp_idx]() {
             std::size_t i = iter_shape.size();
             for (; i > 0; --i)
             {
-                if (ptrdiff_t(temp_idx[i - 1]) >= ptrdiff_t(iter_shape[i - 1]) - 1)
+                if (std::ptrdiff_t(temp_idx[i - 1]) >= std::ptrdiff_t(iter_shape[i - 1]) - 1)
                 {
                     temp_idx[i - 1] = 0;
                 }
@@ -177,19 +210,19 @@ namespace xt
             }
             return std::make_pair(i == 0,
                                   std::inner_product(temp_idx.begin(), temp_idx.end(),
-                                                     iter_strides.begin(), ptrdiff_t(0)));
+                                                     iter_strides.begin(), std::ptrdiff_t(0)));
         };
 
-        auto begin = e.raw_data();
-        auto out = result.raw_data();
-        auto out_begin = result.raw_data();
+        auto begin = e.data();
+        auto out = result.data();
+        auto out_begin = result.data();
 
-        ptrdiff_t next_stride = 0;
+        std::ptrdiff_t next_stride = 0;
 
-        std::pair<bool, ptrdiff_t> idx_res(false, 0);
+        std::pair<bool, std::ptrdiff_t> idx_res(false, 0);
 
         // Remark: eventually some modifications here to make conditions faster where merge + accumulate is the
-        // same function (e.g. check std::is_same<decltype(merge_fct), decltype(acc_fct)>::value) ...
+        // same function (e.g. check std::is_same<decltype(merge_fct), decltype(reduce_fct)>::value) ...
 
         auto merge_border = out;
         bool merge = false;
@@ -200,13 +233,13 @@ namespace xt
         // Decide if going about it row-wise or col-wise
         if (inner_stride == 1)
         {
-            while(idx_res.first != true)
+            while (idx_res.first != true)
             {
                 // for unknown reasons it's much faster to use a temporary variable and
                 // std::accumulate here -- probably some cache behavior
                 result_type tmp;
                 tmp = init_fct(*begin);
-                tmp = std::accumulate(begin + 1, begin + outer_loop_size, tmp, acc_fct);
+                tmp = std::accumulate(begin + 1, begin + outer_loop_size, tmp, reduce_fct);
 
                 // use merge function if necessary
                 *out = merge ? merge_fct(*out, tmp) : tmp;
@@ -231,19 +264,20 @@ namespace xt
         }
         else
         {
-            while(idx_res.first != true)
+            while (idx_res.first != true)
             {
                 std::transform(out, out + inner_loop_size, begin, out,
-                               [merge, &init_fct, &merge_fct](auto&& v1, auto&& v2)
-                               {
-                                    return merge ? merge_fct(v1, v2) : init_fct(v2);
-                               }
-                );
+                               [merge, &init_fct, &merge_fct, &reduce_fct](auto&& v1, auto&& v2) {
+                                    return merge ?
+                                        reduce_fct(v1, v2) :
+                                        // cast because return type of identity function is not upcasted
+                                        static_cast<result_type>(init_fct(v2));
+                               });
 
                 begin += inner_stride;
                 for (std::size_t i = 1; i < outer_loop_size; ++i)
                 {
-                    std::transform(out, out + inner_loop_size, begin, out, acc_fct);
+                    std::transform(out, out + inner_loop_size, begin, out, reduce_fct);
                     begin += inner_stride;
                 }
 
@@ -261,7 +295,6 @@ namespace xt
                 {
                     merge = true;
                 }
-
             };
         }
         return result;
@@ -271,9 +304,6 @@ namespace xt
      * xreducer  *
      *************/
 
-    template <class ST, class X>
-    struct xreducer_shape_type;
-
     template <class REDUCE_FUNC, class INIT_FUNC = xtl::identity, class MERGE_FUNC = REDUCE_FUNC>
     struct xreducer_functors
         : public std::tuple<REDUCE_FUNC, INIT_FUNC, MERGE_FUNC>
@@ -378,7 +408,7 @@ namespace xt
 
         using substepper_type = typename xexpression_type::const_stepper;
         using value_type = std::decay_t<decltype(std::declval<reduce_functor_type>()(
-            std::declval<init_functor_type>()(**(substepper_type*)0), **(substepper_type*)0))>;
+            std::declval<init_functor_type>()(*std::declval<substepper_type>()), *std::declval<substepper_type>()))>;
         using reference = value_type;
         using const_reference = value_type;
         using pointer = value_type*;
@@ -419,7 +449,7 @@ namespace xt
         const_reference element(It first, It last) const;
 
         template <class S>
-        bool broadcast_shape(S& shape) const;
+        bool broadcast_shape(S& shape, bool reuse_cache = false) const;
 
         template <class S>
         bool is_trivial_broadcast(const S& strides) const noexcept;
@@ -449,22 +479,22 @@ namespace xt
     namespace detail
     {
         template <class F, class E, class X>
-        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::lazy) noexcept
+        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::lazy)
         {
             using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<X>>;
             return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
         }
 
         template <class F, class E, class X>
-        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::immediate) noexcept
+        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::immediate)
         {
             return reduce_immediate(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
         }
     }
 
     /**
-     * @brief Returns an \ref xexpression applying the speficied reducing
-     * function to an expresssion over the given axes.
+     * @brief Returns an \ref xexpression applying the specified reducing
+     * function to an expression over the given axes.
      *
      * @param f the reducing function to apply.
      * @param e the \ref xexpression to reduce.
@@ -475,37 +505,35 @@ namespace xt
      * depending on whether \p e is an lvalue or an rvalue.
      */
 
-    template <class F, class E, class X, class ES, class>
-    inline auto reduce(F&& f, E&& e, X&& axes, ES evaluation_strategy) noexcept
+    template <class F, class E, class X, class EVS, class>
+    inline auto reduce(F&& f, E&& e, X&& axes, EVS evaluation_strategy)
     {
         return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes), evaluation_strategy);
     }
 
-    template <class F, class E, class ES, class>
-    inline auto reduce(F&& f, E&& e, ES evaluation_strategy) noexcept
+    template <class F, class E, class EVS, class>
+    inline auto reduce(F&& f, E&& e, EVS evaluation_strategy)
     {
-        auto ar = arange(e.dimension());
+        typename std::decay_t<E>::shape_type ar;
+        resize_container(ar, e.dimension());
+        std::iota(ar.begin(), ar.end(), 0);
         return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ar), evaluation_strategy);
     }
 
 #ifdef X_OLD_CLANG
-    template <class F, class E, class I>
-    inline auto reduce(F&& f, E&& e, std::initializer_list<I> axes) noexcept
+    template <class F, class E, class I, class EVS>
+    inline auto reduce(F&& f, E&& e, std::initializer_list<I> axes, EVS evaluation_strategy)
     {
         using axes_type = std::vector<typename std::decay_t<E>::size_type>;
         using reducer_type = xreducer<F, const_xclosure_t<E>, axes_type>;
-        return reducer_type(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes));
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
     }
 #else
-    template <class F, class E, class I, std::size_t N, class ES>
-    inline auto reduce(F&& f, E&& e, const I (&axes)[N], ES evaluation_strategy) noexcept
+    template <class F, class E, class I, std::size_t N, class EVS>
+    inline auto reduce(F&& f, E&& e, const I (&axes)[N], EVS evaluation_strategy)
     {
         using axes_type = std::array<typename std::decay_t<E>::size_type, N>;
-        // using reducer_type = xreducer<F, const_xclosure_t<E>, axes_type>;
-        // return reducer_type(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
-        auto ar = arange(e.dimension());
         return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
-
     }
 #endif
 
@@ -536,16 +564,16 @@ namespace xt
 
         reference operator*() const;
 
-        void step(size_type dim, size_type n = 1);
-        void step_back(size_type dim, size_type n = 1);
+        void step(size_type dim);
+        void step_back(size_type dim);
+        void step(size_type dim, size_type n);
+        void step_back(size_type dim, size_type n);
         void reset(size_type dim);
         void reset_back(size_type dim);
 
         void to_begin();
         void to_end(layout_type l);
 
-        bool equal(const self_type& rhs) const;
-
     private:
 
         reference aggregate(size_type dim) const;
@@ -555,19 +583,11 @@ namespace xt
         size_type shape(size_type i) const noexcept;
         size_type axis(size_type i) const noexcept;
 
-        const xreducer_type& m_reducer;
+        const xreducer_type* m_reducer;
         size_type m_offset;
         mutable substepper_type m_stepper;
     };
 
-    template <class F, class CT, class X>
-    bool operator==(const xreducer_stepper<F, CT, X>& lhs,
-                    const xreducer_stepper<F, CT, X>& rhs);
-
-    template <class F, class CT, class X>
-    bool operator!=(const xreducer_stepper<F, CT, X>& lhs,
-                    const xreducer_stepper<F, CT, X>& rhs);
-
     /******************
      * xreducer utils *
      ******************/
@@ -647,6 +667,10 @@ namespace xt
         {
             throw std::runtime_error("Reducing axes should be sorted");
         }
+        if (m_axes.size() != 0 && m_axes[m_axes.size() - 1] > m_e.dimension() - 1)
+        {
+            throw std::runtime_error("Axis " + std::to_string(m_axes[m_axes.size() - 1]) + " out of bounds for reduction.");
+        }
         detail::excluding_copy(m_e.shape().cbegin(), m_e.shape().cend(),
                                m_axes.cbegin(), m_axes.cend(),
                                m_shape.begin(), m_dim_mapping.begin());
@@ -706,6 +730,8 @@ namespace xt
     template <class... Args>
     inline auto xreducer<F, CT, X>::operator()(Args... args) const -> const_reference
     {
+        XTENSOR_TRY(check_index(shape(), args...));
+        XTENSOR_CHECK_DIMENSION(shape(), args...);
         std::array<std::size_t, sizeof...(Args)> arg_array = {{static_cast<std::size_t>(args)...}};
         return element(arg_array.cbegin(), arg_array.cend());
     }
@@ -765,11 +791,24 @@ namespace xt
     template <class It>
     inline auto xreducer<F, CT, X>::element(It first, It last) const -> const_reference
     {
+        XTENSOR_TRY(check_element_index(shape(), first, last));
         auto stepper = const_stepper(*this, 0);
         size_type dim = 0;
-        while (first != last)
+        // drop left most elements
+        using difference_type = typename std::iterator_traits<It>::difference_type;
+        auto size = std::ptrdiff_t(dimension()) - std::distance(first, last);
+        auto begin = first - size;
+        while (begin != last)
         {
-            stepper.step(dim++, (std::size_t) *first++);
+            if (begin < first)
+            {
+                stepper.step(dim++, std::size_t(0));
+                begin++;
+            }
+            else
+            {
+                stepper.step(dim++, std::size_t(*begin++));
+            }
         }
         return *stepper;
     }
@@ -786,7 +825,7 @@ namespace xt
      */
     template <class F, class CT, class X>
     template <class S>
-    inline bool xreducer<F, CT, X>::broadcast_shape(S& shape) const
+    inline bool xreducer<F, CT, X>::broadcast_shape(S& shape, bool) const
     {
         return xt::broadcast_shape(m_shape, shape);
     }
@@ -826,7 +865,7 @@ namespace xt
 
     template <class F, class CT, class X>
     inline xreducer_stepper<F, CT, X>::xreducer_stepper(const xreducer_type& red, size_type offset, bool end, layout_type l)
-        : m_reducer(red), m_offset(offset),
+        : m_reducer(&red), m_offset(offset),
           m_stepper(get_substepper_begin())
     {
         if (end)
@@ -842,12 +881,30 @@ namespace xt
         return r;
     }
 
+    template <class F, class CT, class X>
+    inline void xreducer_stepper<F, CT, X>::step(size_type dim)
+    {
+        if (dim >= m_offset)
+        {
+            m_stepper.step(get_dim(dim - m_offset));
+        }
+    }
+
+    template <class F, class CT, class X>
+    inline void xreducer_stepper<F, CT, X>::step_back(size_type dim)
+    {
+        if (dim >= m_offset)
+        {
+            m_stepper.step_back(get_dim(dim - m_offset));
+        }
+    }
+
     template <class F, class CT, class X>
     inline void xreducer_stepper<F, CT, X>::step(size_type dim, size_type n)
     {
         if (dim >= m_offset)
         {
-            m_stepper.step(get_dim(dim), n);
+            m_stepper.step(get_dim(dim - m_offset), n);
         }
     }
 
@@ -856,7 +913,7 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            m_stepper.step_back(get_dim(dim), n);
+            m_stepper.step_back(get_dim(dim - m_offset), n);
         }
     }
 
@@ -865,7 +922,7 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            m_stepper.reset(get_dim(dim));
+            m_stepper.reset(get_dim(dim - m_offset));
         }
     }
 
@@ -874,7 +931,7 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            m_stepper.reset_back(get_dim(dim));
+            m_stepper.reset_back(get_dim(dim - m_offset));
         }
     }
 
@@ -890,34 +947,28 @@ namespace xt
         m_stepper.to_end(l);
     }
 
-    template <class F, class CT, class X>
-    inline bool xreducer_stepper<F, CT, X>::equal(const self_type& rhs) const
-    {
-        return &m_reducer == &(rhs.m_reducer) && m_stepper.equal(rhs.m_stepper);
-    }
-
     template <class F, class CT, class X>
     inline auto xreducer_stepper<F, CT, X>::aggregate(size_type dim) const -> reference
     {
         size_type index = axis(dim);
         size_type size = shape(index);
         reference res;
-        if (dim != m_reducer.m_axes.size() - 1)
+        if (dim != m_reducer->m_axes.size() - 1)
         {
             res = aggregate(dim + 1);
             for (size_type i = 1; i != size; ++i)
             {
                 m_stepper.step(index);
-                res = m_reducer.m_merge(res, aggregate(dim + 1));
+                res = m_reducer->m_merge(res, aggregate(dim + 1));
             }
         }
         else
         {
-            res = m_reducer.m_init(*m_stepper);
+            res = m_reducer->m_init(*m_stepper);
             for (size_type i = 1; i != size; ++i)
             {
                 m_stepper.step(index);
-                res = m_reducer.m_reduce(res, *m_stepper);
+                res = m_reducer->m_reduce(res, *m_stepper);
             }
         }
         m_stepper.reset(index);
@@ -927,39 +978,25 @@ namespace xt
     template <class F, class CT, class X>
     inline auto xreducer_stepper<F, CT, X>::get_substepper_begin() const -> substepper_type
     {
-        return m_reducer.m_e.stepper_begin(m_reducer.m_e.shape());
+        return m_reducer->m_e.stepper_begin(m_reducer->m_e.shape());
     }
 
     template <class F, class CT, class X>
     inline auto xreducer_stepper<F, CT, X>::get_dim(size_type dim) const noexcept -> size_type
     {
-        return m_reducer.m_dim_mapping[dim];
+        return m_reducer->m_dim_mapping[dim];
     }
 
     template <class F, class CT, class X>
     inline auto xreducer_stepper<F, CT, X>::shape(size_type i) const noexcept -> size_type
     {
-        return m_reducer.m_e.shape()[i];
+        return m_reducer->m_e.shape()[i];
     }
 
     template <class F, class CT, class X>
     inline auto xreducer_stepper<F, CT, X>::axis(size_type i) const noexcept -> size_type
     {
-        return m_reducer.m_axes[i];
-    }
-
-    template <class F, class CT, class X>
-    inline bool operator==(const xreducer_stepper<F, CT, X>& lhs,
-                           const xreducer_stepper<F, CT, X>& rhs)
-    {
-        return lhs.equal(rhs);
-    }
-
-    template <class F, class CT, class X>
-    inline bool operator!=(const xreducer_stepper<F, CT, X>& lhs,
-                           const xreducer_stepper<F, CT, X>& rhs)
-    {
-        return !lhs.equal(rhs);
+        return m_reducer->m_axes[i];
     }
 }
 
