@@ -15,6 +15,7 @@
 #include <utility>
 #include <vector>
 
+#include "xbuffer_adaptor.hpp"
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
@@ -28,8 +29,8 @@ namespace xt
     template <class EC, std::size_t N, layout_type L, class Tag>
     struct xcontainer_inner_types<xtensor_container<EC, N, L, Tag>>
     {
-        using container_type = EC;
-        using shape_type = std::array<typename container_type::size_type, N>;
+        using storage_type = EC;
+        using shape_type = std::array<typename storage_type::size_type, N>;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
@@ -68,7 +69,7 @@ namespace xt
         using self_type = xtensor_container<EC, N, L, Tag>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
-        using container_type = typename base_type::container_type;
+        using storage_type = typename base_type::storage_type;
         using allocator_type = typename base_type::allocator_type;
         using value_type = typename base_type::value_type;
         using reference = typename base_type::reference;
@@ -89,7 +90,7 @@ namespace xt
         explicit xtensor_container(const shape_type& shape, const_reference value, layout_type l = L);
         explicit xtensor_container(const shape_type& shape, const strides_type& strides);
         explicit xtensor_container(const shape_type& shape, const strides_type& strides, const_reference value);
-        explicit xtensor_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides);
+        explicit xtensor_container(storage_type&& storage, inner_shape_type&& shape, inner_strides_type&& strides);
 
         template <class S = shape_type>
         static xtensor_container from_shape(S&& s);
@@ -110,10 +111,10 @@ namespace xt
 
     private:
 
-        container_type m_data;
+        storage_type m_storage;
 
-        container_type& data_impl() noexcept;
-        const container_type& data_impl() const noexcept;
+        storage_type& storage_impl() noexcept;
+        const storage_type& storage_impl() const noexcept;
 
         friend class xcontainer<xtensor_container<EC, N, L, Tag>>;
     };
@@ -125,14 +126,14 @@ namespace xt
     template <class EC, std::size_t N, layout_type L, class Tag>
     struct xcontainer_inner_types<xtensor_adaptor<EC, N, L, Tag>>
     {
-        using container_type = std::remove_reference_t<EC>;
-        using shape_type = std::array<typename container_type::size_type, N>;
+        using storage_type = std::remove_reference_t<EC>;
+        using shape_type = std::array<typename storage_type::size_type, N>;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xtensor_container<container_type, N, L, Tag>;
+        using temporary_type = xtensor_container<temporary_container_t<storage_type>, N, L, Tag>;
         static constexpr layout_type layout = L;
     };
 
@@ -168,21 +169,22 @@ namespace xt
         using self_type = xtensor_adaptor<EC, N, L, Tag>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
-        using container_type = typename base_type::container_type;
+        using storage_type = typename base_type::storage_type;
+        using allocator_type = typename base_type::allocator_type;
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
         using backstrides_type = typename base_type::backstrides_type;
         using temporary_type = typename semantic_base::temporary_type;
         using expression_tag = Tag;
 
-        xtensor_adaptor(container_type&& data);
-        xtensor_adaptor(const container_type& data);
+        xtensor_adaptor(storage_type&& storage);
+        xtensor_adaptor(const storage_type& storage);
 
         template <class D>
-        xtensor_adaptor(D&& data, const shape_type& shape, layout_type l = L);
+        xtensor_adaptor(D&& storage, const shape_type& shape, layout_type l = L);
 
         template <class D>
-        xtensor_adaptor(D&& data, const shape_type& shape, const strides_type& strides);
+        xtensor_adaptor(D&& storage, const shape_type& shape, const strides_type& strides);
 
         ~xtensor_adaptor() = default;
 
@@ -198,10 +200,10 @@ namespace xt
 
     private:
 
-        container_closure_type m_data;
+        container_closure_type m_storage;
 
-        container_type& data_impl() noexcept;
-        const container_type& data_impl() const noexcept;
+        storage_type& storage_impl() noexcept;
+        const storage_type& storage_impl() const noexcept;
 
         friend class xcontainer<xtensor_adaptor<EC, N, L, Tag>>;
     };
@@ -219,7 +221,7 @@ namespace xt
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
     inline xtensor_container<EC, N, L, Tag>::xtensor_container()
-        : base_type(), m_data(1, value_type())
+        : base_type(), m_storage(1, value_type())
     {
     }
 
@@ -230,8 +232,8 @@ namespace xt
     inline xtensor_container<EC, N, L, Tag>::xtensor_container(nested_initializer_list_t<value_type, N> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t), true);
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
+        base_type::resize(xt::shape<shape_type>(t), true);
+        L == layout_type::row_major ? nested_copy(m_storage.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
@@ -244,7 +246,7 @@ namespace xt
     inline xtensor_container<EC, N, L, Tag>::xtensor_container(const shape_type& shape, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
@@ -258,8 +260,8 @@ namespace xt
     inline xtensor_container<EC, N, L, Tag>::xtensor_container(const shape_type& shape, const_reference value, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
-        std::fill(m_data.begin(), m_data.end(), value);
+        base_type::resize(shape, l);
+        std::fill(m_storage.begin(), m_storage.end(), value);
     }
 
     /**
@@ -271,7 +273,7 @@ namespace xt
     inline xtensor_container<EC, N, L, Tag>::xtensor_container(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
 
     /**
@@ -285,20 +287,20 @@ namespace xt
     inline xtensor_container<EC, N, L, Tag>::xtensor_container(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
-        base_type::reshape(shape, strides);
-        std::fill(m_data.begin(), m_data.end(), value);
+        base_type::resize(shape, strides);
+        std::fill(m_storage.begin(), m_storage.end(), value);
     }
 
     /**
      * Allocates an xtensor_container by moving specified data, shape and strides
      *
-     * @param data the data for the xtensor_container
+     * @param storage the data for the xtensor_container
      * @param shape the shape of the xtensor_container
      * @param strides the strides of the xtensor_container
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline xtensor_container<EC, N, L, Tag>::xtensor_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides)
-        : base_type(std::move(shape), std::move(strides)), m_data(std::move(data))
+    inline xtensor_container<EC, N, L, Tag>::xtensor_container(storage_type&& storage, inner_shape_type&& shape, inner_strides_type&& strides)
+        : base_type(std::move(shape), std::move(strides)), m_storage(std::move(storage))
     {
     }
 
@@ -328,12 +330,12 @@ namespace xt
         : base_type()
     {
         // Avoids unintialized data because of (m_shape == shape) condition
-        // in reshape (called by assign), which is always true when size() == 1.
+        // in resize (called by assign), which is always true when size() == 1.
         // The condition dimension() == 0 as in xarray is not sufficient because
         // the shape is always initialized since it has a static number of dimensions.
         if (e.derived_cast().size() == 1)
         {
-            m_data.resize(1);
+            detail::resize_data_container(m_storage, std::size_t(1));
         }
         semantic_base::assign(e);
     }
@@ -350,15 +352,15 @@ namespace xt
     //@}
 
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline auto xtensor_container<EC, N, L, Tag>::data_impl() noexcept -> container_type&
+    inline auto xtensor_container<EC, N, L, Tag>::storage_impl() noexcept -> storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline auto xtensor_container<EC, N, L, Tag>::data_impl() const noexcept -> const container_type&
+    inline auto xtensor_container<EC, N, L, Tag>::storage_impl() const noexcept -> const storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     /*******************
@@ -371,52 +373,52 @@ namespace xt
     //@{
     /**
      * Constructs an xtensor_adaptor of the given stl-like container.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(container_type&& data)
-        : base_type(), m_data(std::move(data))
+    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(storage_type&& storage)
+        : base_type(), m_storage(std::move(storage))
     {
     }
 
     /**
      * Constructs an xtensor_adaptor of the given stl-like container.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(const container_type& data)
-        : base_type(), m_data(data)
+    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(const storage_type& storage)
+        : base_type(), m_storage(storage)
     {
     }
 
     /**
      * Constructs an xtensor_adaptor of the given stl-like container,
      * with the specified shape and layout_type.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      * @param shape the shape of the xtensor_adaptor
      * @param l the layout_type of the xtensor_adaptor
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
     template <class D>
-    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(D&& data, const shape_type& shape, layout_type l)
-        : base_type(), m_data(std::forward<D>(data))
+    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(D&& storage, const shape_type& shape, layout_type l)
+        : base_type(), m_storage(std::forward<D>(storage))
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
      * Constructs an xtensor_adaptor of the given stl-like container,
      * with the specified shape and strides.
-     * @param data the container to adapt
+     * @param storage the container to adapt
      * @param shape the shape of the xtensor_adaptor
      * @param strides the strides of the xtensor_adaptor
      */
     template <class EC, std::size_t N, layout_type L, class Tag>
     template <class D>
-    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(D&& data, const shape_type& shape, const strides_type& strides)
-        : base_type(), m_data(std::forward<D>(data))
+    inline xtensor_adaptor<EC, N, L, Tag>::xtensor_adaptor(D&& storage, const shape_type& shape, const strides_type& strides)
+        : base_type(), m_storage(std::forward<D>(storage))
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
     //@}
 
@@ -424,7 +426,7 @@ namespace xt
     inline auto xtensor_adaptor<EC, N, L, Tag>::operator=(const xtensor_adaptor& rhs) -> self_type&
     {
         base_type::operator=(rhs);
-        m_data = rhs.m_data;
+        m_storage = rhs.m_storage;
         return *this;
     }
 
@@ -432,7 +434,7 @@ namespace xt
     inline auto xtensor_adaptor<EC, N, L, Tag>::operator=(xtensor_adaptor&& rhs) -> self_type&
     {
         base_type::operator=(std::move(rhs));
-        m_data = rhs.m_data;
+        m_storage = rhs.m_storage;
         return *this;
     }
 
@@ -442,7 +444,7 @@ namespace xt
         base_type::shape_impl() = std::move(const_cast<shape_type&>(rhs.shape()));
         base_type::strides_impl() = std::move(const_cast<strides_type&>(rhs.strides()));
         base_type::backstrides_impl() = std::move(const_cast<backstrides_type&>(rhs.backstrides()));
-        m_data = std::move(rhs.data());
+        m_storage = std::move(rhs.storage());
         return *this;
     }
 
@@ -462,15 +464,15 @@ namespace xt
     //@}
 
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline auto xtensor_adaptor<EC, N, L, Tag>::data_impl() noexcept -> container_type&
+    inline auto xtensor_adaptor<EC, N, L, Tag>::storage_impl() noexcept -> storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 
     template <class EC, std::size_t N, layout_type L, class Tag>
-    inline auto xtensor_adaptor<EC, N, L, Tag>::data_impl() const noexcept -> const container_type&
+    inline auto xtensor_adaptor<EC, N, L, Tag>::storage_impl() const noexcept -> const storage_type&
     {
-        return m_data;
+        return m_storage;
     }
 }
 
