@@ -53,8 +53,13 @@ namespace xt
     template <class F, class R, class... Args>
     xvectorizer<F, R> vectorize(F&& f, R (*)(Args...));
 
+// Workaround for Visual Studio 15.7.1.
+// Error C2668 (ambiguous call to overloaded function) mistaking a declarations
+// for the definition of another overload.
+#ifndef _MSC_VER
     template <class F>
-    auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr));
+    auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), std::declval<detail::get_function_type<F>*>()));
+#endif
 
     /******************************
      * xvectorizer implementation *
@@ -87,9 +92,9 @@ namespace xt
     }
 
     template <class F>
-    inline auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr))
+    inline auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), std::declval<detail::get_function_type<F>*>()))
     {
-        return vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr);
+        return vectorize(std::forward<F>(f), static_cast<detail::get_function_type<F>*>(nullptr));
     }
 }
 
